package javawrapper;

import junit.framework.*;
import java.io.FileWriter;
import java.io.File;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.InputSource;

/**
 * Tests the BehaviorTypeBase class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestBehaviorTypeBase
    extends TestCase {

  public TestBehaviorTypeBase(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    super.setUp();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  /**
   * Tests XML grid map reading.
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

   */
  public void testGridMapReading() {
    GUIManager oManager = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      String sFileName = WriteXMLGridMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      LightBehaviors oLight = oManager.GetLightBehaviors();
      assertEquals(2, oLight.mp_oGrids[0].mp_oGridVals.size());
      GridValue oVal = (GridValue) oLight.mp_oGrids[0].mp_oGridVals.elementAt(0);
      assertEquals(0, oVal.GetCell().GetX());
      assertEquals(2, oVal.GetCell().GetY());
      assertEquals(12.65, oVal.mp_fFloats[0].floatValue(), 0.0001);
      assertEquals(0, oVal.mp_bBools.length);
      assertEquals(0, oVal.mp_iInts.length);
      assertEquals(0, oVal.mp_sChars.length);

      oVal = (GridValue) oLight.mp_oGrids[0].mp_oGridVals.elementAt(1);
      assertEquals(3, oVal.GetCell().GetX());
      assertEquals(1, oVal.GetCell().GetY());
      assertEquals(45.22, oVal.mp_fFloats[0].floatValue(), 0.0001);
      assertEquals(0, oVal.mp_bBools.length);
      assertEquals(0, oVal.mp_iInts.length);
      assertEquals(0, oVal.mp_sChars.length);

      new File(sFileName).delete();

      //Now load a grid map
      sFileName = WriteXMLGridMap2();
      oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      assertEquals(4, oLight.mp_oGrids[0].mp_oGridVals.size());
      oVal = (GridValue) oLight.mp_oGrids[0].mp_oGridVals.elementAt(2);
      assertEquals(10, oVal.GetCell().GetX());
      assertEquals(11, oVal.GetCell().GetY());
      assertEquals(12.65, oVal.mp_fFloats[0].floatValue(), 0.0001);
      assertEquals(0, oVal.mp_bBools.length);
      assertEquals(0, oVal.mp_iInts.length);
      assertEquals(0, oVal.mp_sChars.length);

      oVal = (GridValue) oLight.mp_oGrids[0].mp_oGridVals.elementAt(3);
      assertEquals(4, oVal.GetCell().GetX());
      assertEquals(9, oVal.GetCell().GetY());
      assertEquals(45.22, oVal.mp_fFloats[0].floatValue(), 0.0001);
      assertEquals(0, oVal.mp_bBools.length);
      assertEquals(0, oVal.mp_iInts.length);
      assertEquals(0, oVal.mp_sChars.length);

      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("XML tree map reading failed with message " + oErr.sMessage);
    }
    catch (org.xml.sax.SAXException oE) {
      fail("Parse test could not parse parameter file - message: " +
           oE.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception processing - bad X cell length
    //We don't validate X and Y cell length anymore because I haven't taken the
    //trouble to distinguish between a user-set change and a grid map-based
    //change
/*    try {
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      String sFileName = WriteXMLGridMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      new File(sFileName).delete();

      //Now add an error file
      sFileName = WriteErrorXMLGridMap1();
      oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      fail("XML grid map reading - bad X cell length was not caught.");

    } catch (ModelException oErr) {
      ;
    }
    catch (org.xml.sax.SAXException oE) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception processing - bad Y cell length
    try {
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      String sFileName = WriteXMLGridMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      new File(sFileName).delete();

      //Now add an error file
      sFileName = WriteErrorXMLGridMap2();
      oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      fail("XML grid map reading - bad Y cell length was not caught.");

    } catch (ModelException oErr) {
      ;
    }
    catch (org.xml.sax.SAXException oE) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    } */

    //Exception processing - bad X plot length
    try {
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      String sFileName = WriteXMLGridMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      new File(sFileName).delete();

      //Now add an error file
      sFileName = WriteErrorXMLGridMap3();
      oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      fail("XML grid map reading - bad X plot length was not caught.");

    } catch (ModelException oErr) {
      ;
    }
    catch (org.xml.sax.SAXException oE) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception processing - bad Y plot length
    try {
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      String sFileName = WriteXMLGridMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      new File(sFileName).delete();

      //Now add an error file
      sFileName = WriteErrorXMLGridMap4();
      oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      fail("XML grid map reading - bad Y plot length was not caught.");

    } catch (ModelException oErr) {
      ;
    }
    catch (org.xml.sax.SAXException oE) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    System.out.println("XML tree map reading test succeeded.");
  }

  /**
   * Writes a tree map file.  There are all four kinds of data members.  This
   * is actually a parameter file with a grid map in it.
   * @return The file name.
   * @throws ModelException
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

   */
  protected String WriteXMLGridMap1() throws ModelException {
    try {
      String sFileName = "\\testgridmap1.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<paramFile fileCode=\"06010101\">");

      //Plot
      oOut.write("<plot>");
      oOut.write("<plot_lenX>200.0</plot_lenX>");
      oOut.write("<plot_lenY>200.0</plot_lenY>");
      oOut.write("<plot_latitude>55.37</plot_latitude>");
      oOut.write("</plot>");

      //Trees - just 'cause I'm not sure we don't need 'em
      oOut.write("<trees>");
      oOut.write("<tr_speciesList>");
      oOut.write("<tr_species speciesName=\"ACSA\"/>");
      oOut.write("<tr_species speciesName=\"ACRU\"/>");
      oOut.write("<tr_species speciesName=\"QURU\"/>");
      oOut.write("<tr_species speciesName=\"PRSE\"/>");
      oOut.write("<tr_species speciesName=\"PIST\"/>");
      oOut.write("<tr_species speciesName=\"TSCA\"/>");
      oOut.write("<tr_species speciesName=\"FRAM\"/>");
      oOut.write("<tr_species speciesName=\"BEAL\"/>");
      oOut.write("<tr_species speciesName=\"FAGR\"/>");
      oOut.write("</tr_speciesList>");
      oOut.write("</trees>");

      //A false GLI grid map
      oOut.write("<grid gridName=\"Quadrat GLI\">");
      oOut.write("<ma_floatCodes>");
      oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
      oOut.write("</ma_floatCodes>");
      oOut.write("<ma_plotLenX>200.000000</ma_plotLenX>");
      oOut.write("<ma_plotLenY>200.000000</ma_plotLenY>");
      oOut.write("<ma_lengthXCells>10.000000</ma_lengthXCells>");
      oOut.write("<ma_lengthYCells>10.000000</ma_lengthYCells>");
      oOut.write("<ma_v x=\"0\" y=\"2\">");
      oOut.write("<fl c=\"0\">12.65</fl>");
      oOut.write("</ma_v>");
      oOut.write("<ma_v x=\"3\" y=\"1\">");
      oOut.write("<fl c=\"0\">45.22</fl>");
      oOut.write("</ma_v>");
      oOut.write("</grid>");
      oOut.write("</paramFile>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      ModelException oErr = new ModelException();
      oErr.sMessage = oE.getMessage();
      throw (oErr);
    }
  }

  /**
   * This writes a fake rundata file with a grid map in it.
   * @return File name of new file.
   * @throws ModelException if there is a problem writing the file.
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

   */
  protected String WriteXMLGridMap2() throws ModelException {
    try {
      String sFileName = "\\testgridmap2.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<timestepRundata fileCode=\"06010701\">");
      oOut.write("<rt_timestep>0</rt_timestep>");

      //A false GLI grid map
      oOut.write("<grid gridName=\"Quadrat GLI\">");
      oOut.write("<ma_floatCodes>");
      oOut.write("<ma_floatCode label=\"GLI\">2</ma_floatCode>");
      oOut.write("</ma_floatCodes>");
      oOut.write("<ma_plotLenX>200.000000</ma_plotLenX>");
      oOut.write("<ma_plotLenY>200.000000</ma_plotLenY>");
      oOut.write("<ma_lengthXCells>10.000000</ma_lengthXCells>");
      oOut.write("<ma_lengthYCells>10.000000</ma_lengthYCells>");
      oOut.write("<ma_v x=\"10\" y=\"11\">");
      oOut.write("<fl c=\"2\">12.65</fl>");
      oOut.write("</ma_v>");
      oOut.write("<ma_v x=\"4\" y=\"9\">");
      oOut.write("<fl c=\"2\">45.22</fl>");
      oOut.write("</ma_v>");
      oOut.write("</grid>");
      oOut.write("</timestepRundata>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      ModelException oErr = new ModelException();
      oErr.sMessage = oE.getMessage();
      throw (oErr);
    }
  }

  /**
   * Writes a grid map that has different X grid length from WriteXMLGridMap2.
   * @return File name of new file.
   * @throws ModelException if there is a problem writing the file.
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

   */
  protected String WriteErrorXMLGridMap1() throws ModelException {
    try {
      String sFileName = "\\testgridmap2.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<timestepRundata fileCode=\"06010701\">");
      oOut.write("<rt_timestep>0</rt_timestep>");

      //A false GLI grid map
      oOut.write("<grid gridName=\"Quadrat GLI\">");
      oOut.write("<ma_floatCodes>");
      oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
      oOut.write("</ma_floatCodes>");
      oOut.write("<ma_plotLenX>200.000000</ma_plotLenX>");
      oOut.write("<ma_plotLenY>200.000000</ma_plotLenY>");
      oOut.write("<ma_lengthXCells>2.000000</ma_lengthXCells>");
      oOut.write("<ma_lengthYCells>10.000000</ma_lengthYCells>");
      oOut.write("<ma_v x=\"11\" y=\"12\">");
      oOut.write("<fl c=\"0\">12.65</fl>");
      oOut.write("</ma_v>");
      oOut.write("<ma_v x=\"5\" y=\"10\">");
      oOut.write("<fl c=\"0\">45.22</fl>");
      oOut.write("</ma_v>");
      oOut.write("</grid>");
      oOut.write("</timestepRundata>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      ModelException oErr = new ModelException();
      oErr.sMessage = oE.getMessage();
      throw (oErr);
    }
  }

  /**
   * Writes a grid map that has different Y grid length from WriteXMLGridMap2.
   * @return File name of new file.
   * @throws ModelException if there is a problem writing the file.
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

   */
  protected String WriteErrorXMLGridMap2() throws ModelException {
    try {
      String sFileName = "\\testgridmap2.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<timestepRundata fileCode=\"06010701\">");
      oOut.write("<rt_timestep>0</rt_timestep>");

      //A false GLI grid map
      oOut.write("<grid gridName=\"Quadrat GLI\">");
      oOut.write("<ma_floatCodes>");
      oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
      oOut.write("</ma_floatCodes>");
      oOut.write("<ma_plotLenX>200.000000</ma_plotLenX>");
      oOut.write("<ma_plotLenY>200.000000</ma_plotLenY>");
      oOut.write("<ma_lengthXCells>10.000000</ma_lengthXCells>");
      oOut.write("<ma_lengthYCells>2.000000</ma_lengthYCells>");
      oOut.write("<ma_v x=\"12\" y=\"13\">");
      oOut.write("<fl c=\"0\">12.65</fl>");
      oOut.write("</ma_v>");
      oOut.write("<ma_v x=\"6\" y=\"11\">");
      oOut.write("<fl c=\"0\">45.22</fl>");
      oOut.write("</ma_v>");
      oOut.write("</grid>");
      oOut.write("</timestepRundata>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      ModelException oErr = new ModelException();
      oErr.sMessage = oE.getMessage();
      throw (oErr);
    }
  }

  /**
   * Writes a grid map that has bad X plot length from WriteXMLGridMap2.
   * @return File name of new file.
   * @throws ModelException if there is a problem writing the file.
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

   */
  protected String WriteErrorXMLGridMap3() throws ModelException {
    try {
      String sFileName = "\\testgridmap2.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<timestepRundata fileCode=\"06010701\">");
      oOut.write("<rt_timestep>0</rt_timestep>");

      //A false GLI grid map
      oOut.write("<grid gridName=\"Quadrat GLI\">");
      oOut.write("<ma_floatCodes>");
      oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
      oOut.write("</ma_floatCodes>");
      oOut.write("<ma_plotLenX>210.000000</ma_plotLenX>");
      oOut.write("<ma_plotLenY>200.000000</ma_plotLenY>");
      oOut.write("<ma_lengthXCells>10.000000</ma_lengthXCells>");
      oOut.write("<ma_lengthYCells>10.000000</ma_lengthYCells>");
      oOut.write("<ma_v x=\"13\" y=\"14\">");
      oOut.write("<fl c=\"0\">12.65</fl>");
      oOut.write("</ma_v>");
      oOut.write("<ma_v x=\"6\" y=\"11\">");
      oOut.write("<fl c=\"0\">45.22</fl>");
      oOut.write("</ma_v>");
      oOut.write("</grid>");
      oOut.write("</timestepRundata>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      ModelException oErr = new ModelException();
      oErr.sMessage = oE.getMessage();
      throw (oErr);
    }
  }

  /**
     * Writes a grid map that has bad Y plot length from WriteXMLGridMap2.
     * @return File name of new file.
     * @throws ModelException if there is a problem writing the file.
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

     */
    protected String WriteErrorXMLGridMap4() throws ModelException {
      try {
        String sFileName = "\\testgridmap2.xml";
        FileWriter oOut = new FileWriter(sFileName);

        oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        oOut.write("<timestepRundata fileCode=\"06010701\">");
        oOut.write("<rt_timestep>0</rt_timestep>");

        //A false GLI grid map
        oOut.write("<grid gridName=\"Quadrat GLI\">");
        oOut.write("<ma_floatCodes>");
        oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
        oOut.write("</ma_floatCodes>");
        oOut.write("<ma_plotLenX>200.000000</ma_plotLenX>");
        oOut.write("<ma_plotLenY>220.000000</ma_plotLenY>");
        oOut.write("<ma_lengthXCells>10.000000</ma_lengthXCells>");
        oOut.write("<ma_lengthYCells>10.000000</ma_lengthYCells>");
        oOut.write("<ma_v x=\"13\" y=\"14\">");
        oOut.write("<fl c=\"0\">12.65</fl>");
        oOut.write("</ma_v>");
        oOut.write("<ma_v x=\"6\" y=\"11\">");
        oOut.write("<fl c=\"0\">45.22</fl>");
        oOut.write("</ma_v>");
        oOut.write("</grid>");
        oOut.write("</timestepRundata>");

        oOut.close();
        return sFileName;
      }
      catch (java.io.IOException oE) {
        ModelException oErr = new ModelException();
        oErr.sMessage = oE.getMessage();
        throw (oErr);
      }
    }

}